@@ -1,6 +1,7 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+import itertools
 import logging
 import os
 import re
@@ -51,7 +52,6 @@ from .progress_stream import StreamOutputError
 from .utils import json_hash
 from .utils import parse_bytes
 from .utils import parse_seconds_float
-from .version import ComposeVersion
 
 
 log = logging.getLogger(__name__)
@@ -172,6 +172,7 @@ class Service(object):
         secrets=None,
         scale=None,
         pid_mode=None,
+        default_platform=None,
         **options
     ):
         self.name = name
@@ -185,13 +186,14 @@ class Service(object):
         self.networks = networks or {}
         self.secrets = secrets or []
         self.scale_num = scale or 1
+        self.default_platform = default_platform
         self.options = options
 
     def __repr__(self):
         return '<Service: {}>'.format(self.name)
 
-    def containers(self, stopped=False, one_off=False, filters={}):
-        filters.update({'label': self.labels(one_off=one_off)})
+    def containers(self, stopped=False, one_off=False, filters={}, labels=None):
+        filters.update({'label': self.labels(one_off=one_off) + (labels or [])})
 
         result = list(filter(None, [
             Container.from_ps(self.client, container)
@@ -202,10 +204,10 @@ class Service(object):
         if result:
             return result
 
-        filters.update({'label': self.labels(one_off=one_off, legacy=True)})
+        filters.update({'label': self.labels(one_off=one_off, legacy=True) + (labels or [])})
         return list(
             filter(
-                self.has_legacy_proj_name, filter(None, [
+                lambda c: c.has_legacy_proj_name(self.project), filter(None, [
                     Container.from_ps(self.client, container)
                     for container in self.client.containers(
                         all=stopped,
@@ -217,9 +219,9 @@ class Service(object):
         """Return a :class:`compose.container.Container` for this service. The
         container must be active, and match `number`.
         """
-        labels = self.labels() + ['{0}={1}'.format(LABEL_CONTAINER_NUMBER, number)]
-        for container in self.client.containers(filters={'label': labels}):
-            return Container.from_ps(self.client, container)
+
+        for container in self.containers(labels=['{0}={1}'.format(LABEL_CONTAINER_NUMBER, number)]):
+            return container
 
         raise ValueError("No container found for %s_%s" % (self.name, number))
 
@@ -256,6 +258,11 @@ class Service(object):
 
         running_containers = self.containers(stopped=False)
         num_running = len(running_containers)
+        for c in running_containers:
+            if not c.has_legacy_proj_name(self.project):
+                continue
+            log.info('Recreating container with legacy name %s' % c.name)
+            self.recreate_container(c, timeout, start_new_container=False)
 
         if desired_num == num_running:
             # do nothing as we already have the desired number
@@ -356,7 +363,16 @@ class Service(object):
 
     @property
     def image_name(self):
-        return self.options.get('image', '{s.project}_{s.name}'.format(s=self))
+        return self.options.get('image', '{project}_{s.name}'.format(
+            s=self, project=self.project.lstrip('_-')
+        ))
+
+    @property
+    def platform(self):
+        platform = self.options.get('platform')
+        if not platform and version_gte(self.client.api_version, '1.35'):
+            platform = self.default_platform
+        return platform
 
     def convergence_plan(self, strategy=ConvergenceStrategy.changed):
         containers = self.containers(stopped=True)
@@ -395,7 +411,7 @@ class Service(object):
         has_diverged = False
 
         for c in containers:
-            if self.has_legacy_proj_name(c):
+            if c.has_legacy_proj_name(self.project):
                 log.debug('%s has diverged: Legacy project name' % c.name)
                 has_diverged = True
                 continue
@@ -704,9 +720,14 @@ class Service(object):
     # TODO: this would benefit from github.com/docker/docker/pull/14699
     # to remove the need to inspect every container
     def _next_container_number(self, one_off=False):
-        containers = self._fetch_containers(
-            all=True,
-            filters={'label': self.labels(one_off=one_off)}
+        containers = itertools.chain(
+            self._fetch_containers(
+                all=True,
+                filters={'label': self.labels(one_off=one_off)}
+            ), self._fetch_containers(
+                all=True,
+                filters={'label': self.labels(one_off=one_off, legacy=True)}
+            )
         )
         numbers = [c.number for c in containers]
         return 1 if not numbers else max(numbers) + 1
@@ -1018,8 +1039,7 @@ class Service(object):
         if not six.PY3 and not IS_WINDOWS_PLATFORM:
             path = path.encode('utf8')
 
-        platform = self.options.get('platform')
-        if platform and version_lt(self.client.api_version, '1.35'):
+        if self.platform and version_lt(self.client.api_version, '1.35'):
             raise OperationFailedError(
                 'Impossible to perform platform-targeted builds for API version < 1.35'
             )
@@ -1044,7 +1064,7 @@ class Service(object):
             },
             gzip=gzip,
             isolation=build_opts.get('isolation', self.options.get('isolation', None)),
-            platform=platform,
+            platform=self.platform,
         )
 
         try:
@@ -1150,14 +1170,14 @@ class Service(object):
         kwargs = {
             'tag': tag or 'latest',
             'stream': True,
-            'platform': self.options.get('platform'),
+            'platform': self.platform,
         }
         if not silent:
             log.info('Pulling %s (%s%s%s)...' % (self.name, repo, separator, tag))
 
         if kwargs['platform'] and version_lt(self.client.api_version, '1.35'):
             raise OperationFailedError(
-                'Impossible to perform platform-targeted builds for API version < 1.35'
+                'Impossible to perform platform-targeted pulls for API version < 1.35'
             )
         try:
             output = self.client.pull(repo, **kwargs)
@@ -1235,12 +1255,6 @@ class Service(object):
 
         return result
 
-    def has_legacy_proj_name(self, ctnr):
-        return (
-            ComposeVersion(ctnr.labels.get(LABEL_VERSION)) < ComposeVersion('1.21.0') and
-            ctnr.project != self.project
-        )
-
 
 def short_id_alias_exists(container, network):
     aliases = container.get(
@@ -1347,7 +1361,7 @@ class ServiceNetworkMode(object):
 
 
 def build_container_name(project, service, number, one_off=False):
-    bits = [project, service]
+    bits = [project.lstrip('-_'), service]
     if one_off:
         bits.append('run')
     return '_'.join(bits + [str(number)])
